@@ -1,6 +1,6 @@
 /*
   zip_get_archive_flag.c -- set archive global flag
-  Copyright (C) 2008-2021 Dieter Baron and Thomas Klausner
+  Copyright (C) 2008-2023 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
   The authors can be contacted at <info@libzip.org>
@@ -57,7 +57,8 @@ zip_set_archive_flag(zip_t *za, zip_flags_t flag, int value) {
         return 0;
     }
 
-    if (ZIP_IS_RDONLY(za)) {
+    /* Allow removing ZIP_AFL_RDONLY if manually set, not if archive was opened read-only. */
+    if (za->flags & ZIP_AFL_RDONLY) {
         zip_error_set(&za->error, ZIP_ER_RDONLY, 0);
         return -1;
     }
