@@ -1,9 +1,9 @@
 /*
   zipcmp.c -- compare zip files
-  Copyright (C) 2003-2021 Dieter Baron and Thomas Klausner
+  Copyright (C) 2003-2024 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
-  The authors can be contacted at <libzip@nih.at>
+  The authors can be contacted at <info@libzip.org>
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -79,6 +79,7 @@ struct entry {
     zip_uint64_t size;
     zip_uint32_t crc;
     zip_uint32_t comp_method;
+    time_t last_modification_time;
     struct ef *extra_fields;
     zip_uint16_t n_extra_fields;
     const char *comment;
@@ -88,123 +89,96 @@ struct entry {
 
 typedef struct {
     uint32_t value;
-    const char * const name;
+    const char *const name;
 } enum_map_t;
 
-const enum_map_t comp_methods[] = {
-    { 0, "Stored (no compression)" },
-    { 1, "Shrunk" },
-    { 2, "Reduced with compression factor 1" },
-    { 3, "Reduced with compression factor 2" },
-    { 4, "Reduced with compression factor 3" },
-    { 5, "Reduced with compression factor 4" },
-    { 6, "Imploded" },
-    { 7, "Reserved for Tokenizing compression algorithm" },
-    { 8, "Deflated" },
-    { 9, "Enhanced Deflating using Deflate64(tm)" },
-    { 10, "PKWARE Data Compression Library Imploding (old IBM TERSE)" },
-    { 11, "11 (Reserved by PKWARE)" },
-    { 12, "BZIP2" },
-    { 13, "13 (Reserved by PKWARE)" },
-    { 14, "LZMA (EFS)" },
-    { 15, "15 (Reserved by PKWARE)" },
-    { 16, "16 (Reserved by PKWARE)" },
-    { 17, "17 (Reserved by PKWARE)" },
-    { 18, "IBM TERSE (new)" },
-    { 19, "IBM LZ77 z Architecture (PFS)" },
-    { 20, "Zstandard compressed data (obsolete)" },
-    { 93, "Zstandard compressed data" },
-    { 95, "XZ compressed data" },
-    { 97, "WavPack compressed data" },
-    { 98, "PPMd version I, Rev 1" },
-    { 99, "WinZIP AES Encryption" },
-    { UINT32_MAX, NULL }
-};
+const enum_map_t comp_methods[] = {{0, "Stored (no compression)"}, {1, "Shrunk"}, {2, "Reduced with compression factor 1"}, {3, "Reduced with compression factor 2"}, {4, "Reduced with compression factor 3"}, {5, "Reduced with compression factor 4"}, {6, "Imploded"}, {7, "Reserved for Tokenizing compression algorithm"}, {8, "Deflated"}, {9, "Enhanced Deflating using Deflate64(tm)"}, {10, "PKWARE Data Compression Library Imploding (old IBM TERSE)"}, {11, "11 (Reserved by PKWARE)"}, {12, "BZIP2"}, {13, "13 (Reserved by PKWARE)"}, {14, "LZMA (EFS)"}, {15, "15 (Reserved by PKWARE)"}, {16, "16 (Reserved by PKWARE)"}, {17, "17 (Reserved by PKWARE)"}, {18, "IBM TERSE (new)"}, {19, "IBM LZ77 z Architecture (PFS)"}, {20, "Zstandard compressed data (obsolete)"}, {93, "Zstandard compressed data"}, {95, "XZ compressed data"}, {97, "WavPack compressed data"}, {98, "PPMd version I, Rev 1"}, {99, "WinZIP AES Encryption"}, {UINT32_MAX, NULL}};
 
 const enum_map_t extra_fields[] = {
     /* PKWARE defined */
-    { 0x0001, "Zip64 extended information" },
-    { 0x0007, "AV Info" },
-    { 0x0008, "Reserved for extended language encoding data (PFS)" },
-    { 0x0009, "OS/2" },
-    { 0x000a, "NTFS" },
-    { 0x000c, "OpenVMS" },
-    { 0x000d, "UNIX" },
-    { 0x000e, "Reserved for file stream and fork descriptors" },
-    { 0x000f, "Patch Descriptor" },
-    { 0x0014, "PKCS#7 Store for X.509 Certificates" },
-    { 0x0015, "X.509 Certificate ID and Signature for individual file" },
-    { 0x0016, "X.509 Certificate ID for Central Directory" },
-    { 0x0017, "Strong Encryption Header" },
-    { 0x0018, "Record Management Controls" },
-    { 0x0019, "PKCS#7 Encryption Recipient Certificate List" },
-    { 0x0065, "IBM S/390 (Z390), AS/400 (I400) attributes - uncompressed" },
-    { 0x0066, "Reserved for IBM S/390 (Z390), AS/400 (I400) attributes - compressed" },
-    { 0x4690, "POSZIP 4690 (reserved)" },
+    {0x0001, "Zip64 extended information"},
+    {0x0007, "AV Info"},
+    {0x0008, "Reserved for extended language encoding data (PFS)"},
+    {0x0009, "OS/2"},
+    {0x000a, "NTFS"},
+    {0x000c, "OpenVMS"},
+    {0x000d, "UNIX"},
+    {0x000e, "Reserved for file stream and fork descriptors"},
+    {0x000f, "Patch Descriptor"},
+    {0x0014, "PKCS#7 Store for X.509 Certificates"},
+    {0x0015, "X.509 Certificate ID and Signature for individual file"},
+    {0x0016, "X.509 Certificate ID for Central Directory"},
+    {0x0017, "Strong Encryption Header"},
+    {0x0018, "Record Management Controls"},
+    {0x0019, "PKCS#7 Encryption Recipient Certificate List"},
+    {0x0065, "IBM S/390 (Z390), AS/400 (I400) attributes - uncompressed"},
+    {0x0066, "Reserved for IBM S/390 (Z390), AS/400 (I400) attributes - compressed"},
+    {0x4690, "POSZIP 4690 (reserved)"},
 
     /* Third-Party defined; see InfoZIP unzip sources proginfo/extrafld.txt */
-    { 0x07c8, "Info-ZIP Macintosh (old)" },
-    { 0x2605, "ZipIt Macintosh (first version)" },
-    { 0x2705, "ZipIt Macintosh 1.3.5+ (w/o full filename)" },
-    { 0x2805, "ZipIt Macintosh 1.3.5+" },
-    { 0x334d, "Info-ZIP Macintosh (new)" },
-    { 0x4154, "Tandem NSK" },
-    { 0x4341, "Acorn/SparkFS" },
-    { 0x4453, "Windows NT security descriptor" },
-    { 0x4704, "VM/CMS" },
-    { 0x470f, "MVS" },
-    { 0x4854, "Theos, old inofficial port" },
-    { 0x4b46, "FWKCS MD5" },
-    { 0x4c41, "OS/2 access control list (text ACL)" },
-    { 0x4d49, "Info-ZIP OpenVMS (obsolete)" },
-    { 0x4d63, "Macintosh SmartZIP" },
-    { 0x4f4c, "Xceed original location extra field" },
-    { 0x5356, "AOS/VS (ACL)" },
-    { 0x5455, "extended timestamp" },
-    { 0x554e, "Xceed unicode extra field" },
-    { 0x5855, "Info-ZIP UNIX (original)" },
-    { 0x6375, "Info-ZIP UTF-8 comment field" },
-    { 0x6542, "BeOS (BeBox, PowerMac, etc.)" },
-    { 0x6854, "Theos" },
-    { 0x7075, "Info-ZIP UTF-8 name field" },
-    { 0x7441, "AtheOS (AtheOS/Syllable attributes)" },
-    { 0x756e, "ASi UNIX" },
-    { 0x7855, "Info-ZIP UNIX" },
-    { 0x7875, "Info-ZIP UNIX 3rd generation" },
-    { 0x9901, "WinZIP AES encryption" },
-    { 0xa220, "Microsoft Open Packaging Growth Hint" },
-    { 0xcafe, "executable Java JAR file" },
-    { 0xfb4a, "SMS/QDOS" }, /* per InfoZIP extrafld.txt */
-    { 0xfd4a, "SMS/QDOS" }, /* per appnote.txt */
-    { UINT32_MAX, NULL }
-};
+    {0x07c8, "Info-ZIP Macintosh (old)"},
+    {0x2605, "ZipIt Macintosh (first version)"},
+    {0x2705, "ZipIt Macintosh 1.3.5+ (w/o full filename)"},
+    {0x2805, "ZipIt Macintosh 1.3.5+"},
+    {0x334d, "Info-ZIP Macintosh (new)"},
+    {0x4154, "Tandem NSK"},
+    {0x4341, "Acorn/SparkFS"},
+    {0x4453, "Windows NT security descriptor"},
+    {0x4704, "VM/CMS"},
+    {0x470f, "MVS"},
+    {0x4854, "Theos, old unofficial port"},
+    {0x4b46, "FWKCS MD5"},
+    {0x4c41, "OS/2 access control list (text ACL)"},
+    {0x4d49, "Info-ZIP OpenVMS (obsolete)"},
+    {0x4d63, "Macintosh SmartZIP"},
+    {0x4f4c, "Xceed original location extra field"},
+    {0x5356, "AOS/VS (ACL)"},
+    {0x5455, "extended timestamp"},
+    {0x554e, "Xceed unicode extra field"},
+    {0x5855, "Info-ZIP UNIX (original)"},
+    {0x6375, "Info-ZIP UTF-8 comment field"},
+    {0x6542, "BeOS (BeBox, PowerMac, etc.)"},
+    {0x6854, "Theos"},
+    {0x7075, "Info-ZIP UTF-8 name field"},
+    {0x7441, "AtheOS (AtheOS/Syllable attributes)"},
+    {0x756e, "ASi UNIX"},
+    {0x7855, "Info-ZIP UNIX"},
+    {0x7875, "Info-ZIP UNIX 3rd generation"},
+    {0x9901, "WinZIP AES encryption"},
+    {0xa220, "Microsoft Open Packaging Growth Hint"},
+    {0xcafe, "executable Java JAR file"},
+    {0xfb4a, "SMS/QDOS"}, /* per InfoZIP extrafld.txt */
+    {0xfd4a, "SMS/QDOS"}, /* per appnote.txt */
+    {UINT32_MAX, NULL}};
 
 
 const char *progname;
 
 #define PROGRAM "zipcmp"
 
-#define USAGE "usage: %s [-hipqtVv] archive1 archive2\n"
+#define USAGE "usage: %s [-" OPTIONS "] archive1 archive2\n"
 
 char help_head[] = PROGRAM " (" PACKAGE ") by Dieter Baron and Thomas Klausner\n\n";
 
 char help[] = "\n\
-  -h       display this help message\n\
   -C       check archive consistencies\n\
+  -h       display this help message\n\
   -i       compare names ignoring case distinctions\n\
   -p       compare as many details as possible\n\
   -q       be quiet\n\
+  -s       print a summary\n\
+  -T       compare time stamps\n\
   -t       test zip files (compare file contents to checksum)\n\
   -V       display version number\n\
   -v       be verbose (print differences, default)\n\
 \n\
-Report bugs to <libzip@nih.at>.\n";
+Report bugs to <info@libzip.org>.\n";
 
 char version_string[] = PROGRAM " (" PACKAGE " " VERSION ")\n\
-Copyright (C) 2003-2021 Dieter Baron and Thomas Klausner\n\
+Copyright (C) 2003-2024 Dieter Baron and Thomas Klausner\n\
 " PACKAGE " comes with ABSOLUTELY NO WARRANTY, to the extent permitted by law.\n";
 
-#define OPTIONS "hVCipqtv"
+#define OPTIONS "ChipqsTtVv"
 
 
 #define BOTH_ARE_ZIPS(a) (a[0].za && a[1].za)
@@ -230,7 +204,8 @@ static int list_directory(const char *name, struct archive *a);
 static int list_zip(const char *name, struct archive *a);
 static int test_file(zip_t *za, zip_uint64_t idx, const char *zipname, const char *filename, zip_uint64_t size, zip_uint32_t crc);
 
-int ignore_case, test_files, paranoid, verbose, have_directory, check_consistency;
+int ignore_case, test_files, paranoid, verbose, have_directory, check_consistency, compare_time_stamps, summary;
+int plus_count = 0, minus_count = 0;
 
 diff_output_t output;
 
@@ -244,43 +219,50 @@ main(int argc, char *const argv[]) {
     ignore_case = 0;
     test_files = 0;
     check_consistency = 0;
+    compare_time_stamps = 0;
     paranoid = 0;
     have_directory = 0;
     verbose = 1;
+    summary = 0;
 
     while ((c = getopt(argc, argv, OPTIONS)) != -1) {
         switch (c) {
-            case 'C':
-                check_consistency = 1;
-                break;
-            case 'i':
-                ignore_case = 1;
-                break;
-            case 'p':
-                paranoid = 1;
-                break;
-            case 'q':
-                verbose = 0;
-                break;
-            case 't':
-                test_files = 1;
-                break;
-            case 'v':
-                verbose = 1;
-                break;
-                
-            case 'h':
-                fputs(help_head, stdout);
-                printf(USAGE, progname);
-                fputs(help, stdout);
-                exit(0);
-            case 'V':
-                fputs(version_string, stdout);
-                exit(0);
-                
-            default:
-                fprintf(stderr, USAGE, progname);
-                exit(2);
+        case 'C':
+            check_consistency = 1;
+            break;
+        case 'h':
+            fputs(help_head, stdout);
+            printf(USAGE, progname);
+            fputs(help, stdout);
+            exit(0);
+        case 'i':
+            ignore_case = 1;
+            break;
+        case 'p':
+            paranoid = 1;
+            break;
+        case 'q':
+            verbose = 0;
+            break;
+        case 's':
+            summary = 1;
+            break;
+        case 'T':
+            compare_time_stamps = 1;
+            break;
+        case 't':
+            test_files = 1;
+            break;
+        case 'V':
+            fputs(version_string, stdout);
+            exit(0);
+        case 'v':
+            verbose = 1;
+            break;
+
+        default:
+            fprintf(stderr, USAGE, progname);
+            exit(2);
         }
     }
 
@@ -340,9 +322,11 @@ compare_zip(char *const zn[]) {
         if (comment_compare(a[0].comment, a[0].comment_length, a[1].comment, a[1].comment_length) != 0) {
             if (a[0].comment_length > 0) {
                 diff_output_data(&output, '-', (const zip_uint8_t *)a[0].comment, a[0].comment_length, "archive comment");
+                minus_count++;
             }
             if (a[1].comment_length > 0) {
                 diff_output_data(&output, '+', (const zip_uint8_t *)a[1].comment, a[1].comment_length, "archive comment");
+                plus_count++;
             }
             res = 1;
         }
@@ -360,6 +344,10 @@ compare_zip(char *const zn[]) {
         free(a[i].entry);
     }
 
+    if (summary) {
+        printf("%d files removed, %d files added\n", minus_count, plus_count);
+    }
+
     switch (res) {
     case 0:
         exit(0);
@@ -421,15 +409,37 @@ list_directory(const char *name, struct archive *a) {
     FTSENT *ent;
     zip_uint64_t nalloc;
     size_t prefix_length;
+    size_t name_length;
+    char* normalized_name;
 
-    char *const names[2] = {(char *)name, NULL};
+    name_length = strlen(name);
+    if (name_length == 0) {
+        fprintf(stderr, "%s: can't open directory '': invalid name\n", progname);
+        return -1;
+    }
 
+    normalized_name = strdup(name);
+
+    while (name_length > 0 && normalized_name[name_length-1] == '/') {
+        name_length -= 1;
+    }
+    normalized_name[name_length] = '\0';
+
+    if (name_length == 0) {
+        normalized_name[0] = '/';
+        normalized_name[1] = '\0';
+        name_length = 1;
+    }
+
+    prefix_length = name_length + 1;
+
+    char *const names[2] = {normalized_name, NULL};
 
     if ((fts = fts_open(names, FTS_NOCHDIR | FTS_LOGICAL, NULL)) == NULL) {
         fprintf(stderr, "%s: can't open directory '%s': %s\n", progname, name, strerror(errno));
+        free(normalized_name);
         return -1;
     }
-    prefix_length = strlen(name) + 1;
 
     nalloc = 0;
 
@@ -437,74 +447,80 @@ list_directory(const char *name, struct archive *a) {
         zip_int64_t crc;
 
         switch (ent->fts_info) {
-            case FTS_DOT:
-            case FTS_DP:
-            case FTS_DEFAULT:
-            case FTS_SL:
-            case FTS_NSOK:
-                break;
+        case FTS_DOT:
+        case FTS_DP:
+        case FTS_DEFAULT:
+        case FTS_SL:
+        case FTS_NSOK:
+            break;
 
-            case FTS_DC:
-            case FTS_DNR:
-            case FTS_ERR:
-            case FTS_NS:
-            case FTS_SLNONE:
-                /* TODO: error */
-                fts_close(fts);
-                return -1;
+        case FTS_DC:
+        case FTS_DNR:
+        case FTS_ERR:
+        case FTS_NS:
+        case FTS_SLNONE:
+            /* TODO: error */
+            fts_close(fts);
+            return -1;
 
-            case FTS_D:
-            case FTS_F:
-                if (a->nentry >= nalloc) {
-                    nalloc += 16;
-                    if (nalloc > SIZE_MAX / sizeof(a->entry[0])) {
-                        fprintf(stderr, "%s: malloc failure\n", progname);
-                        exit(1);
-                    }
-                    a->entry = realloc(a->entry, sizeof(a->entry[0]) * nalloc);
-                    if (a->entry == NULL) {
-                        fprintf(stderr, "%s: malloc failure\n", progname);
-                        exit(1);
-                    }
+        case FTS_D:
+        case FTS_F:
+            if (a->nentry >= nalloc) {
+                nalloc += 16;
+                if (nalloc > SIZE_MAX / sizeof(a->entry[0])) {
+                    fprintf(stderr, "%s: malloc failure\n", progname);
+                    exit(1);
                 }
-                
-                if (ent->fts_info == FTS_D) {
-                    char *dir_name;
-                    
-                    if (ent->fts_path[prefix_length - 1] == '\0') {
-                        break;
-                    }
-                    
-                    dir_name = malloc(strlen(ent->fts_path + prefix_length) + 2);
-                    if (dir_name == NULL) {
-                        fprintf(stderr, "%s: malloc failure\n", progname);
-                        exit(1);
-                    }
-                    sprintf(dir_name, "%s/", ent->fts_path + prefix_length);
-                    a->entry[a->nentry].name = dir_name;
-                    a->entry[a->nentry].size = 0;
-                    a->entry[a->nentry].crc = 0;
+                a->entry = realloc(a->entry, sizeof(a->entry[0]) * nalloc);
+                if (a->entry == NULL) {
+                    fprintf(stderr, "%s: malloc failure\n", progname);
+                    exit(1);
                 }
-                else {
-                    a->entry[a->nentry].name = strdup(ent->fts_path + prefix_length);
-                    a->entry[a->nentry].size = (zip_uint64_t)ent->fts_statp->st_size;
-                    if ((crc = compute_crc(ent->fts_accpath)) < 0) {
-                        fts_close(fts);
-                        return -1;
-                    }
-                
-                    a->entry[a->nentry].crc = (zip_uint32_t)crc;
+            }
+
+            if (ent->fts_info == FTS_D) {
+                char *dir_name;
+
+                if (ent->fts_path[prefix_length - 1] == '\0') {
+                    break;
                 }
-                a->nentry++;
-                break;
+
+                size_t dir_name_size = strlen(ent->fts_path + prefix_length) + 2;
+                dir_name = malloc(dir_name_size);
+                if (dir_name == NULL) {
+                    fprintf(stderr, "%s: malloc failure\n", progname);
+                    exit(1);
+                }
+                snprintf(dir_name, dir_name_size, "%s/", ent->fts_path + prefix_length);
+                a->entry[a->nentry].name = dir_name;
+                a->entry[a->nentry].size = 0;
+                a->entry[a->nentry].crc = 0;
+                a->entry[a->nentry].last_modification_time = 0;
+            }
+            else {
+                a->entry[a->nentry].name = strdup(ent->fts_path + prefix_length);
+                a->entry[a->nentry].size = (zip_uint64_t)ent->fts_statp->st_size;
+                if ((crc = compute_crc(ent->fts_accpath)) < 0) {
+                    fts_close(fts);
+                    free(normalized_name);
+                    return -1;
+                }
+
+                a->entry[a->nentry].crc = (zip_uint32_t)crc;
+                a->entry[a->nentry].last_modification_time = ent->fts_statp->st_mtime;
+            }
+            a->nentry++;
+            break;
         }
     }
 
     if (fts_close(fts)) {
         fprintf(stderr, "%s: error closing directory '%s': %s\n", progname, a->name, strerror(errno));
+        free(normalized_name);
         return -1;
     }
 
+    free(normalized_name);
     return 0;
 }
 #endif
@@ -552,6 +568,7 @@ list_zip(const char *name, struct archive *a) {
                 a->entry[i].comp_method = 0;
                 a->entry[i].n_extra_fields = 0;
             }
+            a->entry[i].last_modification_time = st.mtime;
         }
 
         if (paranoid) {
@@ -584,19 +601,21 @@ comment_compare(const char *c1, size_t l1, const char *c2, size_t l2) {
 }
 
 
-static int compare_list(char *const name[2], const void *list[2], const zip_uint64_t list_length[2], int element_size, int (*cmp)(const void *a, const void *b), int (*ignore)(const void *list, int last, const void *other), int (*check)(char *const name[2], const void *a, const void *b), void (*print)(char side, const void *element), void (*start_file)(const void *element)) {
+static int
+compare_list(char *const name[2], const void *list[2], const zip_uint64_t list_length[2], int element_size, int (*cmp)(const void *a, const void *b), int (*ignore)(const void *list, int last, const void *other), int (*check)(char *const name[2], const void *a, const void *b), void (*print)(char side, const void *element), void (*start_file)(const void *element)) {
     unsigned int i[2];
     int j;
     int diff;
 
 #define INC(k) (i[k]++, list[k] = ((const char *)list[k]) + element_size)
-#define PRINT(k)                                          \
-    do {                                                  \
-        if (ignore && ignore(list[k], i[k] >= list_length[k] - 1, i[1-k] < list_length[1-k] ? list[1-k] : NULL)) {   \
-            break;                                        \
-        }                                                 \
-        print((k) ? '+' : '-', list[k]);                  \
-        diff = 1;                                         \
+#define PRINT(k)                                                                                                         \
+    do {                                                                                                                 \
+        if (ignore && ignore(list[k], i[k] >= list_length[k] - 1, i[1 - k] < list_length[1 - k] ? list[1 - k] : NULL)) { \
+            break;                                                                                                       \
+        }                                                                                                                \
+        print((k) ? '+' : '-', list[k]);                                                                                 \
+        (k) ? plus_count++ : minus_count++;                                                                              \
+        diff = 1;                                                                                                        \
     } while (0)
 
     i[0] = i[1] = 0;
@@ -728,8 +747,12 @@ entry_cmp(const void *p1, const void *p2) {
         else
             return -1;
     }
-    if (e1->crc != e2->crc)
+    if (e1->crc != e2->crc) {
         return (int)e1->crc - (int)e2->crc;
+    }
+    if (compare_time_stamps && e1->last_modification_time != e2->last_modification_time) {
+        return (int)(e1->last_modification_time - e2->last_modification_time);
+    }
 
     return 0;
 }
@@ -739,24 +762,24 @@ static int
 entry_ignore(const void *p, int last, const void *o) {
     const struct entry *e = (const struct entry *)p;
     const struct entry *other = (const struct entry *)o;
-    
+
     size_t length = strlen(e[0].name);
-    
+
     if (length == 0 || e[0].name[length - 1] != '/') {
         /* not a directory */
         return 0;
     }
-    
+
     if (other != NULL && strlen(other->name) > length && strncmp(other->name, e[0].name, length) == 0) {
         /* not empty in other archive */
         return 1;
     }
-    
+
     if (last || (strlen(e[1].name) < length || strncmp(e[0].name, e[1].name, length) != 0)) {
         /* empty in this archive */
         return 0;
     }
-    
+
     /* not empty in this archive */
     return 1;
 }
@@ -792,17 +815,19 @@ entry_paranoia_checks(char *const name[2], const void *p1, const void *p2) {
 }
 
 
-static void entry_print(char side, const void *p) {
+static void
+entry_print(char side, const void *p) {
     const struct entry *e = (struct entry *)p;
 
-    diff_output_file(&output, side, e->name, e->size, e->crc);
+    diff_output_file(&output, side, e->name, e->size, e->crc, e->last_modification_time);
 }
 
 
-static void entry_start_file(const void *p) {
+static void
+entry_start_file(const void *p) {
     const struct entry *e = (struct entry *)p;
-    
-    diff_output_start_file(&output, e->name, e->size, e->crc);
+
+    diff_output_start_file(&output, e->name, e->size, e->crc, e->last_modification_time);
 }
 
 
@@ -848,19 +873,20 @@ test_file(zip_t *za, zip_uint64_t idx, const char *zipname, const char *filename
 }
 
 
-static const char *map_enum(const enum_map_t *map, uint32_t value) {
+static const char *
+map_enum(const enum_map_t *map, uint32_t value) {
     static char unknown[16];
     size_t i = 0;
-    
+
     while (map[i].value < UINT32_MAX) {
         if (map[i].value == value) {
             return map[i].name;
         }
         i++;
     }
-    
+
     snprintf(unknown, sizeof(unknown), "unknown (%u)", value);
     unknown[sizeof(unknown) - 1] = '\0';
-    
+
     return unknown;
 }
