@@ -1,6 +1,6 @@
 /*
   ziptool.c -- tool for modifying zip archive in multiple ways
-  Copyright (C) 2012-2022 Dieter Baron and Thomas Klausner
+  Copyright (C) 2012-2024 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
   The authors can be contacted at <info@libzip.org>
@@ -85,16 +85,12 @@ zip_flags_t stat_flags;
 int hex_encoded_filenames = 0; // Can only be set in ziptool_regress.
 
 static int
-cat_impl(zip_uint64_t idx, zip_uint64_t start, zip_uint64_t len) {
+cat_impl_backend(zip_uint64_t idx, zip_uint64_t start, zip_uint64_t len, FILE *out) {
     zip_error_t error;
     zip_source_t *src;
     zip_int64_t n;
     char buf[8192];
 
-#ifdef _WIN32
-    /* Need to set stdout to binary mode for Windows */
-    setmode(fileno(stdout), _O_BINARY);
-#endif
     zip_error_init(&error);
     /* we can't pass 0 as a len to zip_source_zip_create because it
        will try to give us compressed data */
@@ -125,8 +121,8 @@ cat_impl(zip_uint64_t idx, zip_uint64_t start, zip_uint64_t len) {
         return -1;
     }
     while ((n = zip_source_read(src, buf, sizeof(buf))) > 0) {
-        if (fwrite(buf, (size_t)n, 1, stdout) != 1) {
-            fprintf(stderr, "can't write file contents to stdout: %s\n", strerror(errno));
+        if (fwrite(buf, (size_t)n, 1, out) != 1) {
+            fprintf(stderr, "can't write file contents: %s\n", strerror(errno));
             zip_source_free(src);
             return -1;
         }
@@ -146,6 +142,15 @@ cat_impl(zip_uint64_t idx, zip_uint64_t start, zip_uint64_t len) {
     return 0;
 }
 
+static int
+cat_impl(zip_uint64_t idx, zip_uint64_t start, zip_uint64_t len) {
+#ifdef _WIN32
+    /* Need to set stdout to binary mode for Windows */
+    setmode(fileno(stdout), _O_BINARY);
+#endif
+    return cat_impl_backend(idx, start, len, stdout);
+}
+
 static int
 add(char *argv[]) {
     zip_source_t *zs;
@@ -1055,6 +1060,10 @@ main(int argc, char *argv[]) {
     }
     zip_error_fini(&error);
 
+#ifdef REGRESS_PREPARE_ARGS
+    REGRESS_PREPARE_ARGS
+#endif
+
     err = 0;
     while (arg < argc) {
         int ret;
@@ -1092,24 +1101,37 @@ main(int argc, char *argv[]) {
 #define BIN2HEX(n) ((n) >= 10 ? (n) + 'a' - 10 : (n) + '0')
 #define HEX2BIN(c) (((c) >= '0' && (c) <= '9') ? (c) - '0' : ((c) >= 'A' && (c) <= 'F') ? (c) - 'A' + 10 : (c) - 'a' + 10)
 
-#define FILENAME_BUFFER_LENGTH 8192
+#define FILENAME_BUFFER_LENGTH (2 * 64 * 1024)
 static char filename_buffer[FILENAME_BUFFER_LENGTH + 1];
 
 static const char* encode_filename(const char* name) {
-    if (!hex_encoded_filenames) {
-        return name;
-    }
+    char *t = filename_buffer;
 
-    if (strlen(name) > FILENAME_BUFFER_LENGTH / 2) {
-        // TODO: error message
-        exit(1);
+    if (!hex_encoded_filenames) {
+        const char* s = name;
+        if (strlen(name) > FILENAME_BUFFER_LENGTH) {
+            fprintf(stderr, "internal buffer limit reached, increase buffer size\n");
+            exit(1);
+        }
+        while (*s != '\0') {
+            if (s[0] == '\r' && s[1] == '\n') {
+                s++;
+                continue;
+            }
+            *(t++) = *(s++);
+        }
     }
-    const unsigned char* s = (const unsigned char*)name;
-    char* t = filename_buffer;
-    while (*s != '\0') {
-        *(t++) = BIN2HEX(*s >> 4);
-        *(t++) = BIN2HEX(*s & 0xf);
-        s += 1;
+    else {
+        const unsigned char *s = (const unsigned char *)name;
+        if (strlen(name) > FILENAME_BUFFER_LENGTH / 2) {
+            fprintf(stderr, "internal buffer limit reached, increase buffer size\n");
+            exit(1);
+        }
+        while (*s != '\0') {
+            *(t++) = BIN2HEX(*s >> 4);
+            *(t++) = BIN2HEX(*s & 0xf);
+            s += 1;
+        }
     }
     *t = '\0';
     return filename_buffer;
@@ -1121,7 +1143,7 @@ static const char* decode_filename(const char* name) {
     }
 
     if (strlen(name) > FILENAME_BUFFER_LENGTH * 2) {
-        // TODO: error message
+        fprintf(stderr, "internal buffer limit reached, increase buffer size\n");
         exit(1);
     }
     // TODO: check that strlen(name) % 2 == 0
@@ -1136,4 +1158,4 @@ static const char* decode_filename(const char* name) {
     *t = '\0';
 
     return filename_buffer;
-}
\ No newline at end of file
+}
